%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%    BINARY ENDOGENOUS TREATMENT    %%%%%%%%
%%%%%%%%%   STOCHASTIC FRONTIER MODEL   %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This file replicates the empirical application in 
%%% Centorrino, Perez-Urdiales, Ureta, and Wall (2023)
%%% Author: Samuele Centorrino
%%% Email: samuele.centorrino@stonybrook.edu
%%% Last update: July 2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear

%% Optimization options
% Number of starting points for the optimization algorithm
nmulti  = 120;
foptunc= optimoptions('fminunc','Algorithm','quasi-newton','Display','off',...
               'MaxFunctionEvaluations',1e+10,'MaxIterations',1e+10,'OptimalityTolerance',2.2204e-16);
           
foptcon= optimoptions('fmincon','Algorithm','interior-point','Display','off',...
                'MaxFunctionEvaluations',1e+10,'MaxIterations',1e+10,'OptimalityTolerance',2.2204e-16);

%% IMPORT DATA FROM TEXT FILE
% Name of variables    
rng(123456789)

TABL   = readtable('elsalvadordata.txt');
names  = TABL.Properties.VariableNames;

% Permute database to avoid 0s and 1s to be clustered
permvec= sample((1:size(TABL,1))',size(TABL,1),0);
TABL   = TABL(permvec,:);
N      = size(TABL,1);

x1     = [TABL.lland./std(TABL.lland),TABL.llabour./std(TABL.llabour),...
            TABL.lfertilizer./std(TABL.lfertilizer),...
            TABL.lpesticide./std(TABL.lpesticide),TABL.lseeds./std(TABL.lseeds)];
        
z1     = [TABL.lland./std(TABL.lland),...
            TABL.fullten,...
            TABL.fullten.*TABL.lland./std(TABL.lland),...
            TABL.ageold60,...
            TABL.maxeduc./std(TABL.maxeduc),...
            TABL.nooutinc,...
            TABL.footaccess,...
            TABL.caraccess,...
            -log(TABL.riskmax)./std(log(TABL.riskmax))];

% Assign variables to regression objects
y     = TABL.loutput;
    
x     = [x1,x1.*repmat(TABL.Participation,1,size(x1,2)),...
             TABL.Participation];
z     = [z1,z1.*repmat(TABL.Participation,1,size(z1,2)),...
            TABL.Participation];

% NUMBER  OF COMPONENTS NOT INCLUDED IN FIRST STAGE
nendx = size(x1,2) + 1;
nendz = size(z1,2) + 1;

w1      = [TABL.diste1 >= 3.5,TABL.valorjornal,...
                TABL.fam_electricidad,TABL.fam_letrinas];
w       = [w1,TABL.dum02,TABL.dum03,TABL.dum04];

EXV     = [ones(N,1) x(:,1:(size(x,2)-nendx)) z(:,2:(size(z,2)-nendz)) w];
        
% TWO STEP ESTIMATION
% First stage and test instruments
probest = probit(z(:,end),EXV,10000);
probest1= probit(z(:,end),[ones(N,1) x(:,1:(size(x,2)-nendx)) z(:,2:(size(z,2)-nendz))],10000);
indins  = (length(probest.beta)-size(w,2) + 1):length(probest.beta);
waldtest= probest.beta(indins)'*(probest.covb(indins,indins)\probest.beta(indins));
lrtestw = 2*(probest.lik - probest1.lik);

%% Test normality of residuals (Wilde, 2008)
Zhat = normcdf(EXV*probest.beta);
sres = (z(:,end) - Zhat)./sqrt(Zhat.*(1 - Zhat));

% Augmented regression
Xhat = [(1/3).*(1-(EXV*probest.beta).^2) 0.25.*((EXV*probest.beta).*(3 +...
                    (EXV*probest.beta).^2)) EXV];
evar = (normpdf(EXV*probest.beta)./sqrt(Zhat.*(1 - Zhat))).*Xhat;

lmrtest =sres'*(evar * ((evar'*evar) \evar')) * sres;
pvalspec= 1-chi2cdf(lmrtest,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% ESTIMATION OF SF MODEL WITHOUT CONTROLLING FOR ENDOGENEITY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
olsest  = ols(y,[ones(N,1) x]);
sbeta   = olsest.beta;

errols  = y - [ones(N,1) x]*sbeta;

sdelta  = zeros(size(z,2),1);
ss      = [var(errols);0.2];

mxfunex = @(u) liksf(u,y,[ones(N,1) x],z,0);

stval   = [sbeta;sdelta;ss];
stvalrnd= stval + (-0.25 + 0.5*rand(length(stval),nmulti));
gradvec = {};
hmat    = {};

mxparex = zeros(length(stval),nmulti);
lkvalex = zeros(nmulti,1);

defaultProfile = parallel.defaultClusterProfile;
myCluster = parcluster(defaultProfile);

%% Open parallel pool here
parpool(myCluster,myCluster.NumWorkers);
parfor l = 1:nmulti
    while isinf(mxfunex(stvalrnd(:,l))) || isnan(mxfunex(stvalrnd(:,l))) 
        stvalrnd(:,l) = stval + (-0.5 + rand(length(stval),1));
        1
    end
    [mxparex(:,l),lkvalex(l),~,~,gradvec{l},hmat{l}] = fminunc(mxfunex,stvalrnd(:,l),foptunc);
end

totsize      = size(x,2) + size(z,2);
[~,ind1]     = min(lkvalex.*(~isinf(abs(lkvalex))));
fparex       = mxparex(:,ind1);
deltahatex   = fparex((size(x,2)+2):(totsize + 1));
sv2hatex     = fparex(totsize + 2);
su2hatex     = fparex(totsize + 3);
betahatex    = fparex(1:(size(x,2)+1));

hessianmatex = hmat{ind1};
coefcovex    = pinv(hessianmatex);
   
% Std Errors and 95% Confidence Intervals
stderrex     = sqrt(diag(coefcovex));

% Profile Likelihood confidence intervals (more robust to deviations from
% normality)
lowbfn       = @(u,v,indv) (2*mxfunex(fparex) - 2*mxfunex([u(1:(indv-1));v;u(indv:length(u))]) + chi2inv(0.95,1)).^2 + v;
upbfn        = @(u,v,indv) (2*mxfunex(fparex) - 2*mxfunex([u(1:(indv-1));v;u(indv:length(u))]) + chi2inv(0.95,1)).^2 - v;
ciparex      = zeros(length(fparex),2);
parfor kk = 1:length(fparex)
    R0ci    = zeros(1,length(fparex));
    R0ci(kk)=1;
    indkk    = [1:(kk-1) (kk+1):length(fparex)];
    tlbound = fmincon(@(t) lowbfn(t(indkk),t(kk),kk),fparex,R0ci,fparex(kk),[],[],[],[],[],foptcon);
    tubound = fmincon(@(t) upbfn(t(indkk),t(kk),kk),fparex,-R0ci,-fparex(kk),[],[],[],[],[],foptcon);
    ciparex(kk,:) = [tlbound(kk) tubound(kk)];
end

epsex    = y - [ones(N,1) x]*behatex;

% Technical inefficiency using Battese-Coelli formula
tehatex  = teex(fparex,y,x,z);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% NOW CONTROL FOR POSSIBLY ENDOGENOUS EFFICIENCY DETERMINANTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sgamma  = probest.beta;  
sbeta   = olsest.beta;
if ~isreal(stderrex)
    sdelta  = zeros(size(z,2)+1,1);
else
    sdelta  = [0.5*log(su2hatex);deltahatex];
end
srho    = [0;0];
ss2     = sv2hatex;

stval   = [sbeta;sdelta;srho;ss2];
mxfun   = @(u) likendsftreat(u,y,[ones(N,1) x],z,EXV);

stvalrnd= [sbeta + 0.25*(-1 + 2*rand(length(sbeta),nmulti));...
            sdelta + 0.25*(-1 + 2*rand(length(sdelta),nmulti));...
            0.5*(-1 + 2*rand(1,nmulti));...
            0.5*rand(1,nmulti);...
            exp(log(ss2) + 0.25*(-1 + 2*rand(length(ss2),nmulti)));...
            repmat(sgamma,1,nmulti)]; 
boundw   = EXV*sgamma;

% Easier to impose constraints on the parameters using lower and upper
% bound instead of linear equality/inequality constraints
lb       = [-Inf*ones((size(x,2) + size(z,2) + 3),1);0;-Inf*ones(length(sgamma) + 1,1)]; 
ub       = [Inf*ones((size(x,2) + size(z,2) + 3),1);0.99;Inf*ones(length(sgamma) + 1,1)]; 
ubres    = [Inf*ones((size(x,2) + size(z,2) + 3),1);0;Inf*ones(length(sgamma) + 1,1)]; 
R0       = [zeros((size(x,2) + size(z,2) + 3),1);1;zeros(length(sgamma) + 1,1)]; 

gradvec  = {};
hmat     = {};
mxpar    = zeros(size(stvalrnd,1),nmulti);
lkval    = zeros(nmulti,1);

gradvecres= {};
hmatres   = {};
mxparres  = zeros(size(stvalrnd,1),nmulti);
lkvalres  = zeros(nmulti,1);

parfor l = 1:nmulti
    tmpstrval= stvalrnd(:,l);
    while isinf(mxfun(stvalrnd(:,l))) || isnan(mxfun(stvalrnd(:,l)))
        stvalrnd(:,l) = [sbeta;...
                        sdelta + 0.5*(-1 + 2*rand(length(sdelta),1));...
                        0.5*(-1 + 2*rand(1));...
                        rand(1);...
                        exp(log(ss2) + 0.25*(-1 + 2*rand(length(ss2),1)));...
                        sgamma];
        tmpstrval= stvalrnd(:,l);
         
        disp(l)
    end
    [mxpar(:,l),lkval(l),~,~,~,gradvec{l},hmat{l}] = fmincon(mxfun,stvalrnd(:,l),[],[],[],[],lb,ub,[],foptcon);
    [mxparres(:,l),lkvalres(l),~,~,~,gradvecres{l},hmatres{l}] = fmincon(mxfun,(eye(length(R0)) - R0*R0')*stvalrnd(:,l),[],[],[],[],lb,ubres,[],foptcon);
end
[~,ind1]     = min(lkval.*(~isinf(abs(lkval))));
[~,ind2]     = min(lkvalres.*(~isinf(abs(lkval))));
fpar         = mxpar(:,ind1);
fparres      = mxparres(:,ind2);

%% TEST THE NULL THAT RHO_U==0
testlr = -2*(lkval(ind1) - lkvalres(ind2));
ctval2 = chi2inv(0.9,1);
ctval1 = chi2inv(0.4/0.5,1);

%% Test based on the score as in Bottai (2003) and Ekwall and Bottai (2022)
dstep     = 1e-04;
augvec    = zeros(N,length(fparres));
for i = 1:N
    for kk = 1:length(fpar)
        iotavec     = zeros(length(fpar),1);
        iotavec(kk) = 1;
        augvec(i,kk)= 0.5*(likendsftreat(fparres + dstep.*iotavec,y(i),[1 x(i,:)],z(i,:),EXV(i,:)) -...
                        likendsftreat(fparres - dstep.*iotavec,y(i),[1 x(i,:)],z(i,:),EXV(i,:)))./dstep;
    end
end
sderrho      = sderliktreatrho0(fparres,y,[ones(N,1) x],z,EXV);
modgrad      = augvec;
modgrad(:,size(x,2) + size(z,2) + 4)= sderrho;    
testhmat     = (1/N)*(cov(modgrad)\eye(length(fparres)));
testscr      = sum(modgrad,1)*testhmat*sum(modgrad,1)';
            
%% STORE RESULTS
hessianmat   = hmat{ind1};
hessianmatres= hmatres{ind2};

betahat      = fpar(1:(size(x,2)+1));
deltahat     = fpar((size(x,2) + 3):(size(x,2) + size(z,2) + 2));
rhovhat      = fpar(size(x,2) + size(z,2) + 3);
rhouhat      = fpar(size(x,2) + size(z,2) + 4);
sv2hat       = fpar(size(x,2) + size(z,2) + 5);
su2hat       = exp(2*fpar(size(x,2) + 2));
gammahat     = fpar((size(x,2) + size(z,2) + 6):(size(x,2) + size(z,2) + 5 + size(EXV,2)));

% Compute numerical hessian for standard errors and define the first stage
% error term
tsigmav      = sqrt((1 - rhovhat^2)*sv2hat);
sigmau0      = sqrt(su2hat).*exp(z(:,1:end-nendz)*deltahat(1:end-nendz));
sigmau1      = exp(fpar(size(x,2) + 1) + z(:,1:end-1)*deltahat(1:end-1) + deltahat(end));
tsigmau      = sqrt((1 - rhouhat^2)).*exp(fpar(size(x,2) + 1) + z*deltahat);
lambda       = real(tsigmau./tsigmav);
sigma2       = tsigmav.^2 + tsigmau.^2;
sigma        = sqrt(sigma2);
coefcov      = pinv(hessianmat);

stderrp      = sqrt(diag(coefcov));

eps          = y - [ones(N,1) x]*betahat;

%% EKVALL AND BOTTAI (AoS, 2022)
% Confidence intervals based on the inversion of the score test
% The starting value is taken to be the unconstrained parameter estimate as
% the when rhou=0 is selected as starting value a numerical algorirthm will
% get stuck into that local minimum
ciparend      = zeros(length(fparres),2);
parfor kk = 1:length(fparres)
    gridlower = linspace(fparres(kk) - 2.5*stderrp(kk),fparres(kk),50)';
    gridupper = linspace(fparres(kk),fparres(kk) + 2.5*stderrp(kk),50)';

    if kk == size(x,2) + size(z,2) + 4
        tlbound=0;
    else
        vfunlow = zeros(50,1);
        for ll = 1:50
            vfunlow(ll) = scoreci(gridlower(ll),fpar,@likendsftreat,y,x,z,EXV,kk);
        end
        [~,indlow] = min(vfunlow);
        tlbound    = gridlower(indlow);
    end
    vfunup  = zeros(50,1);
    for ll = 1:50
        vfunup(ll) = scoreci(gridupper(ll),fpar,@likendsftreat,y,x,z,EXV,kk);
    end
    [~,indup]= min(vfunup);
    tubound  = gridupper(indup);
    
    ciparend(kk,:) = [tlbound tubound];
end
delete(gcp) %Parallel pool closes here
ciparend(size(x,2) + 2,:) = exp(2*ciparend(size(x,2) + 2,:));

%% Technical efficiency
%% UNRESTRICTED MODEL
tehat      = teendbin(fpar,y,x,z);

%% RESTRICTED MODEL
tehatres   = teendbin(fparres,y,x,z);

%% STOCHASTIC DOMINANCE TEST AS IN LINTON ET AL. (2005)
scr0ex  = tehatex(z(:,end)==0);
scr0end = tehat(z(:,end)==0);

scr1ex  = tehatex(z(:,end)==1);
scr1end = tehat(z(:,end)==1);

% For participants, we want to test that the distribution of TE under
% exogeneity first order stochastically dominate the distribution of TE
% under endogeneity
% The opposite for nonparticipants
neval = 100;
grte  = linspace(0.1,1,neval)';

% Z = 0
n0        = sum(z(:,end)==0);

% Test statistic for FSD
Dhat10ex  = mean(scr0ex <= grte',1);
Dhat10end = mean(scr0end <= grte',1);
dstat10   = max(sqrt(n0)*(Dhat10ex - Dhat10end));

% Z = 1
n1        = sum(z(:,end)==1);

% Test statistic for FSD
Dhat11ex  = mean(scr1ex <= grte',1);
Dhat11end = mean(scr1end <= grte',1);
dstat11   = max(sqrt(n1)*(Dhat11end - Dhat11ex));

% Subsample to obtain critical values
bn = (55:10:425)';

% Z = 0
critval10 = zeros(length(bn),1);
critval11 = zeros(length(bn),1);
parpool
parfor jj =  1:length(bn)
    dstat10ss = zeros(N-bn(jj)+1,1);
    dstat11ss = zeros(N-bn(jj)+1,1);
    for ii = 1:(N-bn(jj)+1)
      
        indss        = sample((1:N)',bn(jj),0);
        % Exogenous estimator
        tmpmxfunex   = @(u) liksf(u,y(indss),[ones(bn(jj),1) x(indss,:)],z(indss,:),0);
        tmpmxparex   = fminunc(tmpmxfunex,fparex,foptunc);
    
        tehatexss    = teex(tmpmxparex,y(indss),x(indss,:),z(indss,:));
    
        % Endogenous estimator
        tmpmxfun     = @(u) likendsftreat(u,y(indss),[ones(bn(jj),1) x(indss,:)],...
                                    z(indss,:),EXV(indss,:));
        tmpmxpar     = fmincon(tmpmxfun,fpar,[],[],[],[],lb,ub,[],foptcon);
        tehatss      = teendbin(tmpmxpar,y(indss),x(indss,:),z(indss,:));
    
        scr0exss     = tehatexss(z(indss,end)==0);
        scr1exss     = tehatexss(z(indss,end)==1);
        scr0endss    = tehatss(z(indss,end)==0);
        scr1endss    = tehatss(z(indss,end)==1);
    
        n0ss         = sum(z(indss,end)==0);
        n1ss         = sum(z(indss,end)==1);
    
        Dhat10exss   = mean(scr0exss <= grte',1);
        Dhat10endss  = mean(scr0endss <= grte',1);
        dstat10ss(ii)= max(sqrt(n0ss)*(Dhat10exss - Dhat10endss));
      
        Dhat11exss   = mean(scr1exss <= grte',1);
        Dhat11endss  = mean(scr1endss <= grte',1);
        dstat11ss(ii)= max(sqrt(n1ss)*(Dhat11endss - Dhat11exss));
    end
    critval10(jj) = quantile(dstat10ss,0.95);
    critval11(jj) = quantile(dstat11ss,0.95);
end
delete(gcp)

cv10sub = median(critval10);
cv11sub = median(critval11);

clear defaultProfile myCluster
save('ElSalvador_data.mat')