%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%    BINARY ENDOGENOUS TREATMENT    %%%%%%%%
%%%%%%%%%   STOCHASTIC FRONTIER MODEL   %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% This file replicates the simulations in 
%%% Centorrino, Perez-Urdiales, Ureta, and Wall (2023)
%%% Author: Samuele Centorrino
%%% Email: samuele.centorrino@stonybrook.edu
%%% Last update: June 2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
rng(1234567)

%%%%%%%%%%%% 1 sample of (y,z,w,x) = data
simnum = 1000;
N      = obs(ii);  
nmulti = 10;

% Parameters
mu    = zeros(1,5);
ccoef = 0.5;
si    = toeplitz([1,repmat(ccoef,1,4)]);

s2u   = pi/(pi-2);
s2v   = 1;
alpha = 0;
beta  = sqrt(1/(3+9*sqrt(0.1)));
pix   = sqrt(0.1);
piz   = sqrt(0.1);
delta1= 0;
delta2= 0;

% Correlation matrix between errors
if thrcorr == 1
    rhou = 0;
elseif thrcorr == 2
    rhou = 0.5;
else
    rhou = 0.95;
end
rhov    = 0.5;

wx  = mvnrnd(mu, si, N*simnum);
x1  = reshape(wx(:,1),N,simnum);
x2  = reshape(wx(:,2),N,simnum);
z1  = reshape(wx(:,3),N,simnum);
w1  = reshape(wx(:,4),N,simnum);
w2  = reshape(wx(:,5) > 0.5,N,simnum);

% Generate correlation between errors
% (in first simulation scheme X is not endogenous)
eta1  = normrnd(0,1,N,simnum);
u0    = abs(sqrt(s2u)*(rhou*eta1 + sqrt(1 - rhou.^2)*normrnd(0,1,N,simnum)));

% % %% DEFINE THE TRUNCATED NORMAL/NORMAL DISTRIBUTION
% jpdf    = @(u1,u2) (2./(2*pi.*sqrt((1-rhou.^2).*s2u))).*exp(-0.5*(u1.^2 ...
%                 - 2*rhou*sqrt(s2u).*u1.*u2 + s2u*u2.^2)./((1-rhou.^2).*s2u));
% mpdfu   = @(u1) (2./(sqrt(2*pi*s2u))).*exp(-0.5*(u1.^2)./s2u);
% mpdfeta = @(u2) (2./(sqrt(2*pi))).*exp(-0.5*u2.^2).*normcdf(rhou*u2/sqrt(1 -rhou.^2));
% ueval1  = linspace(0,9,100)';
% ueval2  = linspace(-4,5,100)';
% fneval  = jpdf(repmat(ueval1',100,1),repmat(ueval2,1,100));
% 
% conducdf= @(u1,u2) jpdf(u1,u2)./mpdfeta(u2);
% integral(@(u) conducdf(u,1),0,Inf)
% 
% %Plot joint distribution
% surf(ueval1,ueval2,fneval)
% %Plot the marginal distributions
% figure
% histogram(etastar(:),'Normalization','pdf')
% hold on
% plot(ueval2,mpdfeta(ueval2),'k','Linewidth',2)
% saveas(gcf,'ETA_DISTR.pdf')
% close
% 
% figure
% histogram(u0(:),'Normalization','pdf')
% hold on
% plot(ueval1,mpdfu(ueval1),'k','Linewidth',2)
% xlim([0 10])
% saveas(gcf,'U_DISTR.pdf')
% close
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
v   = sqrt(s2v)*(rhov*eta1 + sqrt(1 - rhov.^2)*normrnd(0,1,N,simnum));

z2  = double(-0.1 + piz*(x1 + x2 + z1 + w1) + w2 + eta1 >= 0);

u   = u0.*exp(z1*delta1 + z2*delta2); 
y   = alpha + x1*beta + x2*beta + (x1.*z2)*beta + (x2.*z2)*beta + v - u;

%% Preallocate objects
% Exogeneous
betahatex  = zeros(simnum,5);
deltahatex = zeros(simnum,2);
sv2hatex   = zeros(simnum,1);
su2hatex   = zeros(simnum,1);

% Kumbhakar et al. 2009
betahatkum = zeros(simnum,5);
deltahatkum= zeros(simnum,2);
sv2hatkum  = zeros(simnum,1);
su2hatkum  = zeros(simnum,1);
rhohatkum  = zeros(simnum,2);
gammahatkum= zeros(simnum,length(mu) + 1);

% Endogenous
betahat  = zeros(simnum,5);
gammahat = zeros(simnum,length(mu) + 1);
deltahat = zeros(simnum,2);
rhohat   = zeros(simnum,2);
sv2hat   = zeros(simnum,1);
su2hat   = zeros(simnum,1);
testlr   = zeros(simnum,1);
testscr  = zeros(simnum,1);

% STORE VALUE OF THE LIKELIHOOD FUNCTION AT MINIMUM
lrtestw = zeros(simnum,1);
fvallik = zeros(simnum,1);

tehat    = zeros(N,simnum);

defaultProfile = parallel.defaultClusterProfile;
myCluster = parcluster(defaultProfile);
if myCluster.NumWorkers >= 40
    myCluster.NumWorkers = 32;
end

foptunc  = optimoptions('fminunc','Algorithm','quasi-newton','Display','off',...
               'MaxFunctionEvaluations',1e+10,'MaxIterations',1e+10,'OptimalityTolerance',2.2204e-16);
           
foptcon  = optimoptions('fmincon','Algorithm','interior-point','Display','off',...
                'MaxFunctionEvaluations',1e+10,'MaxIterations',1e+10,'OptimalityTolerance',2.2204e-16);

% Critical values for tests
ctval2   = chi2inv([0.90 0.95 0.99],1);
ctval1   = 0.5 + 0.5*ctval2;

% True parameter vector
tpar     = [alpha beta beta beta beta 0.5*log(s2u) delta1 delta2 rhov rhou s2v -0.1 repmat(piz,1,length(mu)-1) 1]';

% Save restricted parameter value for testing
fparres  = zeros(simnum,length(tpar));
sderrex  = zeros(simnum,8);
sderrend = zeros(simnum,length(tpar));
sderrendres= zeros(simnum,length(tpar));
rankhes    = zeros(simnum,1);

parpool(myCluster,myCluster.NumWorkers);
parfevalOnAll(@warning,0,'off','all')
parfor j = 1:simnum
    xx     = [x1(:,j) x2(:,j) x1(:,j).*z2(:,j) x2(:,j).*z2(:,j)];     

    zz     = [z1(:,j) z2(:,j)];
    ww     = [w1(:,j) w2(:,j)]; 

    EXV    = [ones(N,1) x1(:,j) x2(:,j) z1(:,j) ww];

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% ESTIMATION OF SF MODEL WITHOUT CONTROLLING FOR ENDOGENEITY
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    olsest  = ols(y(:,j),[ones(N,1) xx]);

    sdelta  = [0;0];
    ss      = [0;0];
    sbeta   = olsest.beta(2:5);

    mxfunex = @(u) liksf(u,y(:,j),xx,zz,0);

    stval   = [sbeta;sdelta;exp(ss)];
    stvalrnd= stval + (-1 + 2*rand(length(stval),nmulti));

    mxparex = zeros(length(stval),nmulti);
    lkvalex = zeros(nmulti,1);
    hmatex  = {};
    for l = 1:nmulti
        while isinf(mxfunex(stvalrnd(:,l))) || isnan(mxfunex(stvalrnd(:,l))) || ~isreal(mxfunex(stvalrnd(:,l))) 
            stvalrnd(:,l) = stval + (-1 + 2*rand(length(stval),1));
            1
        end
        [mxparex(:,l),lkvalex(l),~,~,~,hmatex{l}] = fminunc(mxfunex,stvalrnd(:,l),foptunc);
    end

    [~,ind1]       = min(lkvalex.*(~isinf(abs(lkvalex))));
    fparex         = mxparex(:,ind1);
    sderrex(j,:)   = sqrt(diag(inv(hmatex{ind1})));
    
    deltahatex(j,:)= fparex((size(xx,2)+1):(size(xx,2) + size(zz,2)));
    sv2hatex(j)    = fparex(size(xx,2) + size(zz,2) + 1);
    su2hatex(j)    = fparex(size(xx,2) + size(zz,2) + 2);
    alphatex       = mean(y(:,j) - xx*fparex(1:size(xx,2)) + sqrt(2/pi)*sqrt(su2hatex(j)).*exp(zz*deltahatex(j,:)'));
    betahatex(j,:) = [alphatex;fparex(1:size(xx,2))];

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% ESTIMATION OF SF MODEL CONTROLLING FOR ENDOGENEITY
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    probest   = probit(z2(:,j),EXV,1000);
    probest1  = probit(z2(:,j),[ones(N,1) xx z1(:,j)],1000);
    lrtestw(j)= 2*(probest.lik - probest1.lik);

    sgamma   = probest.beta;
    sdelta   = [0.5*log(su2hatex(j));0;0];
    srho     = [0;0];
    ss2      = sv2hatex(j);
    sbeta    = betahatex(j,:)';

    mxfun    = @(u) likendsftreat(u,y(:,j),[ones(N,1) xx],zz,EXV);

    stval    = [sbeta;sdelta;srho;ss2;sgamma];
    stvalrnd = [repmat(sbeta,1,nmulti);...
                sdelta + 0.25*(-1 + 2*rand(length(sdelta),nmulti));...
                srho(1) + 0.25*(-1 + 2*rand(1,nmulti));...
                rand(1,nmulti);...
                exp(log(ss2) + 0.25*(-1 + 2*rand(length(ss2),nmulti)));...
                repmat(sgamma,1,nmulti)];  
    boundw   = EXV*sgamma;

    % Restrict rhoU = 0
    R0       = -[zeros(1,(size(xx,2) + size(zz,2) + 3)),1,zeros(1,size(EXV,2) + 1)]; 
    % Restrict rhoV = 0 (for comparison with Kumbhakar et al, 2009)
    R1       = -[zeros(1,(size(xx,2) + size(zz,2) + 2)),1,zeros(1,size(EXV,2) + 2)]; 
    gradvec  = {};
    gradvecres0= {};
    gradvecres1= {};
    hmat     = {};
    hmatres0 = {};
    hmatres1 = {};
    mxpar    = zeros(size(stvalrnd,1),nmulti);
    mxparres0= zeros(size(stvalrnd,1),nmulti);
    mxparres1= zeros(size(stvalrnd,1),nmulti);

    lkval    = zeros(nmulti,1);
    lkvalres0= zeros(nmulti,1);
    lkvalres1= zeros(nmulti,1);
    for l = 1:nmulti
        while mxfun(stvalrnd(:,l)) == 1e+10*N || isinf(mxfun(stvalrnd(:,l)))...
                        || isnan(mxfun(stvalrnd(:,l))) || ~isreal(mxfun(stvalrnd(:,l)))
            stvalrnd(:,l) = [sbeta;...
                            sdelta + 0.25*(-1 + 2*rand(length(sdelta),1));...
                            0.25*(-1 + 2*rand(1,1));...
                            rand(1,1);...
                            exp(log(ss2) + 0.25*(-1 + 2*rand(length(ss2),1)));...
                            sgamma];  
                        1
        end
        [mxpar(:,l),lkval(l),~,~,~,gradvec{l},hmat{l}] = fmincon(mxfun,stvalrnd(:,l),...
                                                                R0,0,[],[],[],[],[],foptcon);
        [mxparres0(:,l),lkvalres0(l),~,~,~,gradvecres0{l},hmatres0{l}]= fmincon(mxfun,(eye(length(R0)) - R0'*R0)*stvalrnd(:,l),...
                                                                [],[],R0,0,[],[],[],foptcon);
        [mxparres1(:,l),lkvalres1(l),~,~,~,gradvecres1{l},hmatres1{l}]= fmincon(mxfun,(eye(length(R1)) - R1'*R1)*stvalrnd(:,l),...
                                                                R0,0,R1,0,[],[],[],foptcon);
    end
    [~,ind1]     = min(lkval.*(~isinf(abs(lkval))));
    [~,ind2]     = min(lkvalres0.*(~isinf(abs(lkvalres0))));
    [~,ind3]     = min(lkvalres1.*(~isinf(abs(lkvalres1))));
    fpar         = mxpar(:,ind1);
    tfparres     = mxparres0(:,ind2);
    fvallik(j)   = lkval(ind1);
    fparres(j,:) = tfparres;
    
    betahat(j,:) = fpar(1:(size(xx,2)+1));
    deltahat(j,:)= fpar((size(xx,2) + 3):(size(xx,2) + size(zz,2) + 2));
    rhohat(j,:)  = fpar((size(xx,2) + size(zz,2) + 3):(size(xx,2) + size(zz,2) + 4));
    sv2hat(j)    = fpar(size(xx,2) + size(zz,2) + 5);
    su2hat(j)    = exp(2*fpar(size(xx,2) + 2));
    gammahat(j,:)= fpar(size(xx,2) + size(zz,2) + 6:end);
    
    % Kumbhakar et al. (2009)
    fparkum         = mxparres1(:,ind3);
    betahatkum(j,:) = fparkum(1:(size(xx,2)+1));
    deltahatkum(j,:)= fparkum((size(xx,2) + 3):(size(xx,2) + size(zz,2) + 2));
    rhohatkum(j,:)  = fparkum((size(xx,2) + size(zz,2) + 3):(size(xx,2) + size(zz,2) + 4));
    sv2hatkum(j)    = fparkum(size(xx,2) + size(zz,2) + 5);
    su2hatkum(j)    = exp(2*fparkum(size(xx,2) + 2));
    gammahatkum(j,:)= fparkum(size(xx,2) + size(zz,2) + 6:end);

    % Keep numerical approximation of jacobian and hessian
    jacobmax     = gradvec{ind1};
    coefcov      = pinv(hmat{ind1});
    coefcov(isnan(coefcov) | isinf(coefcov)) = 0;
    
    % Check the rank of the Hessian matrix
    [~,S]        = eig( hmatres0{ind2} );
    rankhes(j)   = rank(S);
    sderrend(j,:)= sqrt(diag(coefcov));
        
    %% Likelihood ratio test as in Andrews (2001)
    testlr(j)    = -2*(lkval(ind1) - lkvalres0(ind2));

    %% Modified score test as in Bottai (2003) and Ekwall and Bottai (2022)
    augvec    = zeros(N,length(tfparres));
    for i = 1:N
        [~,~,~,~,augvec(i,:)]= fminunc(@(u) likendsftreat(u,y(i,j),[1 xx(i,:)],zz(i,:),EXV(i,:)),...
                            tfparres,optimoptions('fminunc','Algorithm','quasi-newton','Display','off',...
                            'MaxFunctionEvaluations',0,'MaxIterations',0,'OptimalityTolerance',2.2204e-16));
    end    
    sderrho     = sderliktreatrho0(tfparres,y(:,j),[ones(N,1) xx],zz,EXV);
    modgrad     = augvec;
    modgrad(:,size(xx,2) + size(zz,2) + 4)= sderrho;
    testhmat    = (1/N)*pinv(cov(modgrad));
    
    testscr(j)  = sum(modgrad,1)*testhmat*sum(modgrad,1)';
    
    %% COMPUTE INEFFICIENCY SCORES
    tehat(:,j)  = teendbin(fpar,y(:,j),xx,zz);
        
    disp(j)
end
delete(gcp)

MSEbeta = mean((betahat - ones(simnum,1)*[alpha beta beta beta beta]).^2,1);
MSEdelta= mean((deltahat - ones(simnum,1)*[delta1 delta2]).^2,1);
MSEsv   = mean((sv2hat - ones(simnum,1)*s2v).^2,1);
MSEsu   = mean((su2hat - ones(simnum,1)*s2u).^2,1);
MSErho  = mean((rhohat - ones(simnum,1)*[rhov rhou]).^2,1);
MSEgam  = mean((gammahat - ones(simnum,1)*[0 piz*ones(1,4) 1]).^2,1);

% Size/Power
% LR Test
rejproblr = mean(repmat(testlr,1,3) >= repmat(ctval1,simnum,1),1);
% SCR Test
rejprobscr= mean(repmat(testscr,1,3) >= repmat(ctval2,simnum,1),1);

save(['Binary_End_frontier_',num2str(N),'_',num2str(thrcorr),'.mat'])