 function valfn = likendsftreat(par,y,x,z,w)
    %% Estimation of a SFM with endogenous treatment
    % Centorrino et al. (2023, Journal of Applied Econometrics)
    % Arguments:
    % par = vector of estimated parameters from liksf.m
    % y   = Nx1 vector, dependent variable 
    % x   = N x p matrix, inputs (not including the constant term)
    % z   = N x q matrix, environmental variables
    % w   = N x d matrix, instrumental variables    
    % Author: Samuele Centorrino
    % Email: samuele.centorrino@stonybrook.edu

    N       = length(y);
     
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%% THETA: Parameters of the log-likelihood function %%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    beta    = par(1:size(x,2));
    delta   = par((size(x,2) + 1):(size(x,2) + size(z,2) + 1));
    rho     = par((size(x,2) + size(z,2) + 2):(size(x,2) + size(z,2) + 3));
    s2      = par((size(x,2) + size(z,2) + 4));
    gamma   = par((size(x,2) + size(z,2) + 5):end);
     
    % Define objects for function
    rhov    = rho(1);
    rhou    = rho(2);
     
    sigmav  = sqrt(s2);
    tsigmav = sqrt((1 - rhov.^2))*sigmav;
    
    %%%%% Index for first stage regression
    boundw  = w*gamma;

    %%%% Values of parameters
    sigmau0 = exp(delta(1) + z*delta(2:end));
    tsigmau0= sqrt(1 - rhou.^2).*sigmau0;

    sigma20 = tsigmav.^2 + tsigmau0.^2;
    lambda0 = tsigmau0./tsigmav;
    sigma0  = sqrt(sigma20);
    
    tcorr0p = rhov*sigmav - rhou*sigmau0;
    tcorr0n = rhov*sigmav + rhou*sigmau0;

    s2eps0p = sigmau0.^2 + sigmav.^2 - 2*rhov*sigmav*rhou*sigmau0;
    s2eps0n = sigmau0.^2 + sigmav.^2 + 2*rhov*sigmav*rhou*sigmau0;
     
    %%% Conditional mean and variance
    q0p       = lambda0.*rhov*sigmav./sigma0 + (rhou.*sigmau0)./(lambda0.*sigma0);
    q0n       = lambda0.*rhov*sigmav./sigma0 - (rhou.*sigmau0)./(lambda0.*sigma0);
    
    cmean0p   = tcorr0p./s2eps0p;
    cmean0n   = tcorr0n./s2eps0n;
 
    s2eta0p   = sigma20./s2eps0p;
    s2eta0n   = sigma20./s2eps0n;

    s2tau0p   = 1 + (q0p.^2).*s2eta0p;
    s2tau0n   = 1 + (q0n.^2).*s2eta0n;
        
    % CHECK IF MATRIX IS POSITIVE DEFINITE
    vnormf0p  = sqrt(s2tau0p);
    
    %elem10p   = abs(q0p.*sqrt(s2eta0p)./vnormf0p);
    eigm10p   = (s2eta0p + 1)/2 + sqrt((s2eta0p + 1).^2/4 - s2eta0p./s2tau0p);
    eigm20p   = (s2eta0p + 1)/2 - sqrt((s2eta0p + 1).^2/4 - s2eta0p./s2tau0p);
    
    % CHECK IF MATRIX IS POSITIVE DEFINITE
    vnormf0n  = sqrt(s2tau0n);
    
    %elem10n   = abs(q0n.*sqrt(s2eta0n)./vnormf0n);
    eigm10n    = (s2eta0n + 1)/2 + sqrt((s2eta0n + 1).^2/4 - s2eta0n./s2tau0n);
    eigm20n    = (s2eta0n + 1)/2 - sqrt((s2eta0n + 1).^2/4 - s2eta0n./s2tau0n);

    % TOTAL SUM OF FLAGS
    flag01p    = sum(~isreal(sqrt(eigm10p))) + sum(~isreal(sqrt(eigm20p))) ...
                    + sum(isnan(eigm10p)) + sum(isnan(eigm20p)) + sum(~isreal(vnormf0p)) + sum(~isreal(q0p));
    flag01n    = sum(~isreal(sqrt(eigm10n))) + sum(~isreal(sqrt(eigm20n))) ...
                    + sum(isnan(eigm10n)) + sum(isnan(eigm20n)) + sum(~isreal(vnormf0n)) + sum(~isreal(q0n));
    
    totflag    = flag01p + flag01n;
    
    if abs(rhou) < 1-1e-5 && abs(rhov) < 1-1e-5 && min(tsigmau0) > 1e-20 && sum(~isreal(sigma0)) == 0 ...
            && isreal(sigmav) && sum(totflag)==0

        % Define the error term 
        deps0p   = (-lambda0./sigma0 + cmean0p.*q0p)./vnormf0p;        
        deps0n   = (-lambda0./sigma0 + cmean0n.*q0n)./vnormf0n;
        
        eps      = y - x*beta;
        
        %%% Conditional probabilities
        % POSITIVE SIGN
        %% ZEROS
     	covmat0p = [ones(N,1), -q0p.*sqrt(s2eta0p)./vnormf0p,...
                       -q0p.*sqrt(s2eta0p)./vnormf0p, ones(N,1)];
        part10p  = (1./sqrt(s2eps0p)).*normpdf(eps./sqrt(s2eps0p)); 
        matval0p = [deps0p.*eps ,(-boundw - cmean0p.*eps)./sqrt(s2eta0p)];
        part20p  = zeros(N,1);
        for ii = 1:N
            part20p(ii) = mvncdf(matval0p(ii,:),[ ],reshape(covmat0p(ii,:),2,2));
        end

        %% ONES
        part21p = normcdf(deps0p.*eps) - part20p;

        % NEGATIVE SIGN
        %% ZEROS
     	covmat0n = [ones(N,1), -q0n.*sqrt(s2eta0n)./vnormf0n,...
                       -q0n.*sqrt(s2eta0n)./vnormf0n, ones(N,1)];
        part10n  = (1./sqrt(s2eps0n)).*normpdf(eps./sqrt(s2eps0n));
        matval0n = [deps0n.*eps ,(-boundw - cmean0n.*eps)./sqrt(s2eta0n)];
        part20n  = zeros(N,1);
        for ii = 1:N
            part20n(ii) = mvncdf(matval0n(ii,:),[],reshape(covmat0n(ii,:),2,2));
        end
                
        %% ONES
        part21n  = normcdf(deps0n.*eps) - part20n;  

        % CHECK PROBABILITIES
        if min(part21p) < 0 || min(part21n) < 0
            warning('Probabilities out of bounds 0')
        elseif max(part21p) > 1 || max(part21n) > 1
            warning('Probabilities out of bounds 1')
        end
        
        part21p(part21p < 0) = 0;
        part21n(part21n < 0) = 0;
        
        part21p(part21p > 1) = 1;
        part21n(part21n > 1) = 1;
        
        %% Likelihood function
        lkpart1  = part21p.*part10p + part21n.*part10n;
        lkpart0  = part20p.*part10p + part20n.*part10n;
        
        totlik   = (lkpart1.^z(:,end)) .* (lkpart0.^(1-z(:,end)));
        totlik(totlik <= 0) = 1e-100;

        tempvalfn = -sum(log(totlik));
        
        if isnan(tempvalfn) || isinf(tempvalfn)
            valfn   = Inf;
        else
            valfn   = tempvalfn;
        end
    else
        valfn   = Inf;
    end
end