function valfn = liksf(par,y,x,z,model,varargin)
    %% Estimation of a SFM with linear frontier and scale exponential inefficiency
    % Arguments
    % par = vector of estimated parameters from liksf.m
    % y   = Nx1 vector, dependent variable 
    % x   = N x p matrix, inputs (not including the constant term)
    % z   = N x q matrix, environmental variables
    % model = dummy variable 
    %           0 THE INEFFICIENCY TERM FOLLOWS A HALF-NORMAL DISTRIBUTION
    %           1 THE INEFFICIENCY TERM FOLLOWS A NORMAL DISTRIBUTION TRUNCATED
    %           AT 0, AND WITH MEAN MU
    % varargin = weights for the likelihood function
    % Author: Samuele Centorrino
    % Email: samuele.centorrino@stonybrook.edu
    
    N   = length(y);
    
    if ~isempty(varargin)
        weights = varargin{1};
    else
        weights = ones(N,1);
    end
    
    if min(par((size(x,2) + 1 + size(z,2)):end)) < 0
        valfn = Inf;
    else
        if model == 0
            
            beta    = par(1:size(x,2));  
            delta   = par((size(x,2) + 1):(size(x,2) + size(z,2)));
            vareps  = par((size(x,2) + 1 + size(z,2)):end);
    
            sigmav  = sqrt(vareps(1));
    
            % Values of parameters
            if isempty(delta)
                sigmau  = sqrt(vareps(2));
            else
                sigmau  = sqrt(vareps(2)).*exp(z*delta);
            end
            lambda  = sigmau./sigmav;
    
            sigma2  = sigmav.^2 + sigmau.^2;
            sigma   = sqrt(sigma2);
            
            eps     = y - x*beta;
    
            part11  = -log(normpdf(eps./sigma));
            part21  = -log(normcdf(-(lambda.*eps)./sigma));
            
            if length(sigma2) == 1
                part31  = repmat(log(sigma),N,1);
            else
                part31  = log(sigma);
            end
            %valfn   = sum(part31) - sum(part21) + sum(part11);
            
        elseif model == 1
            
            beta    = par(1:size(x,2));
            mu      = par(size(x,2) + size(z,2) + 1);
            vareps  = par((size(x,2) + 1 + size(z,2)):end);
            delta   = par((size(x,2) + 1):(size(x,2) + size(z,2)));
            sigmav  = sqrt(vareps(1));
            
            % Values of parameters
            sigmau  = sqrt(vareps(2))*exp(z*delta);
            lambda  = sigmau./sigmav;
    
            sigma2  = sigmav.^2 + sigmau.^2;
            sigma   = sqrt(sigma2);
    
            alpha   = mean(y - x*beta + mu*exp(z*delta) + sqrt(2/pi)*sigmau);
            eps     = y - alpha - x*beta;
    
            part11  = log(normpdf((eps + mu.*exp(z*delta))./sigma));
            part21  = log(real(normcdf(mu./(lambda.*sigma)-(lambda.*eps)./sigma)));
            part31  = N*log(normcdf(mu./vareps(2)));
        end
    
        if sigmav >= 0 && vareps(2) >= 0 && sigmav^2 <= 0.5*(max(y) - min(y)).^2 ...
                    && vareps(2) <= 0.5*(max(y) - min(y)).^2
            valfn  = sum(weights.*(part11 + part21 + part31));
        else
            valfn  = Inf;
        end
    end
end